// The MIT License
// 
// Copyright (c) 2011 Tim Smart
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

// Base controller constructor
var Base = function Base (app, name, alias) {
  this.app   = app
  this.name  = name
  this.views = name + '/'

  // Render param alias
  this.alias = ['title', 'title_name']
  this.alias.push.apply(this.alias, alias)
}

exports.Base = Base

Base.prototype.setupRequest = function (request, response, next) {
  return new BaseRequest(request, response, next)
}

// Contructor for storing render params etc
var BaseRequest = function BaseRequest (request, response, next) {
  this.request  = request
  this.response = response
  this.next     = next

  // Default values
  this.data     = BaseRequest.DEFAULT_DATA
}

exports.BaseRequest = BaseRequest

BaseRequest.DEFAULT_DATA =
  { title      : 'express-helpers'
  , title_name : ''
  }

BaseRequest.prototype.setTitle = function (title) {
  return this.data.title = title
}

BaseRequest.prototype.render = function (file, params, external) {
  if (!external) {
    file = this.views + file
  }

  params || (params = {})

  var data = {}
    , keys = Object.keys(params)
    , key

  for (var i = 0, il = keys.length; i < il; i++) {
    key            = keys[i]
    this.data[key] = params[key]
  }

  return res.render(file, this.data)
}

// Controller loader / cache
var cache    = exports.cache    = {}
  , fn_cache = exports.fn_cache = {}

exports.app = null

exports.loadController = function (name) {
  var controller

  if (cache[name]) {
    return cache[name]
  }

  controller = new (require('./' + name))(exports.app)

  return cache[name] = controller
}

exports.load = function (name, method) {
  var controller, fn_key

  method || (method = 'index')

  fn_key = name + ':' + method

  if (fn_cache[fn_key]) {
    return fn_cache[fn_key]
  } else if (cache[name]) {
    controller = cache[name]
  } else {
    controller = new (require('./' + name))(exports.app)
    cache[name] = controller
  }

  return fn_cache[fn_key] = function () {
    controller[method].apply(controller, arguments)
  }
}
